"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MappedExceptionFilter = void 0;
const common_1 = require("@nestjs/common");
const grpc_1 = require("grpc");
const rxjs_1 = require("rxjs");
const typeorm_1 = require("typeorm");
const util_1 = require("util");
const mapped_exception_error_class_1 = require("../mapped-exception-error.class");
var ErrorLayerEnum;
(function (ErrorLayerEnum) {
    ErrorLayerEnum[ErrorLayerEnum["DATABASE"] = 0] = "DATABASE";
    ErrorLayerEnum[ErrorLayerEnum["VALIDATION"] = 1] = "VALIDATION";
    ErrorLayerEnum[ErrorLayerEnum["APPLICATION"] = 2] = "APPLICATION";
    ErrorLayerEnum[ErrorLayerEnum["EXCEPTION"] = 3] = "EXCEPTION";
})(ErrorLayerEnum || (ErrorLayerEnum = {}));
const RpcStatusToHttpStatusMap = {
    [grpc_1.status.OK]: common_1.HttpStatus.OK,
    [grpc_1.status.CANCELLED]: common_1.HttpStatus.SERVICE_UNAVAILABLE,
    [grpc_1.status.UNKNOWN]: common_1.HttpStatus.INTERNAL_SERVER_ERROR,
    [grpc_1.status.INVALID_ARGUMENT]: common_1.HttpStatus.BAD_REQUEST,
    [grpc_1.status.DEADLINE_EXCEEDED]: common_1.HttpStatus.INTERNAL_SERVER_ERROR,
    [grpc_1.status.NOT_FOUND]: common_1.HttpStatus.NOT_FOUND,
    [grpc_1.status.ALREADY_EXISTS]: common_1.HttpStatus.CONFLICT,
    [grpc_1.status.PERMISSION_DENIED]: common_1.HttpStatus.FORBIDDEN,
    [grpc_1.status.UNAUTHENTICATED]: common_1.HttpStatus.UNAUTHORIZED,
    [grpc_1.status.RESOURCE_EXHAUSTED]: common_1.HttpStatus.TOO_MANY_REQUESTS,
    [grpc_1.status.FAILED_PRECONDITION]: common_1.HttpStatus.PRECONDITION_FAILED,
    [grpc_1.status.ABORTED]: common_1.HttpStatus.INTERNAL_SERVER_ERROR,
    [grpc_1.status.OUT_OF_RANGE]: common_1.HttpStatus.BAD_REQUEST,
    [grpc_1.status.UNIMPLEMENTED]: common_1.HttpStatus.NOT_IMPLEMENTED,
    [grpc_1.status.UNAVAILABLE]: common_1.HttpStatus.SERVICE_UNAVAILABLE,
    [grpc_1.status.DATA_LOSS]: common_1.HttpStatus.INTERNAL_SERVER_ERROR,
};
const HttpStatusToRpcStatusMap = {
    [common_1.HttpStatus.BAD_REQUEST]: grpc_1.status.INVALID_ARGUMENT,
    [common_1.HttpStatus.UNAUTHORIZED]: grpc_1.status.UNAUTHENTICATED,
    [common_1.HttpStatus.FORBIDDEN]: grpc_1.status.PERMISSION_DENIED,
    [common_1.HttpStatus.NOT_FOUND]: grpc_1.status.NOT_FOUND,
    [common_1.HttpStatus.TOO_MANY_REQUESTS]: grpc_1.status.RESOURCE_EXHAUSTED,
    [common_1.HttpStatus.BAD_GATEWAY]: grpc_1.status.UNKNOWN,
    [common_1.HttpStatus.SERVICE_UNAVAILABLE]: grpc_1.status.UNAVAILABLE,
    [common_1.HttpStatus.OK]: grpc_1.status.OK,
    [common_1.HttpStatus.INTERNAL_SERVER_ERROR]: grpc_1.status.INTERNAL,
    [common_1.HttpStatus.NOT_IMPLEMENTED]: grpc_1.status.UNIMPLEMENTED,
};
let MappedExceptionFilter = (() => {
    let MappedExceptionFilter = class MappedExceptionFilter {
        constructor() {
            this.defaultApplicationErrorPrefix = process.env.EXCEPTION_ERROR_PREFIX || 'APPLICATION_ERR';
            this.defaultDatabaseErrorPrefix = process.env.EXCEPTION_DATABASE_ERROR_PREFIX || 'DATABASE_ERR';
            this.defaultValidationErrorPrefix = process.env.EXCEPTION_VALIDATION_ERROR_PREFIX || 'VALIDATION_ERR';
        }
        catch(exception, host) {
            const ctx = host.switchToHttp();
            const response = ctx.getResponse();
            const request = ctx.getRequest();
            const contextType = host.getType();
            const { status = common_1.HttpStatus.INTERNAL_SERVER_ERROR, message, code, } = this.getStatusAndMessage(exception, contextType);
            if (contextType === 'http') {
                return this.handleRestContext(request, response, status, message, code);
            }
            if (contextType === 'rpc') {
                return this.handleRpcContext(code, message, status);
            }
            if (contextType === 'graphql') {
                return this.graphqlFormatError(status, message, code);
            }
        }
        getStatusAndMessage(exception, contextType) {
            var _a, _b, _c, _d;
            const errorLayer = this.identifyErrorLayer(exception);
            if (errorLayer === ErrorLayerEnum.DATABASE) {
                return {
                    code: this.defaultDatabaseErrorPrefix,
                    message: this.extractMessageFromException(exception, contextType),
                    status: this.extractStatusCodeFromException(exception, common_1.HttpStatus.INTERNAL_SERVER_ERROR),
                };
            }
            if (errorLayer === ErrorLayerEnum.VALIDATION) {
                let message;
                if ((_a = exception === null || exception === void 0 ? void 0 : exception.response) === null || _a === void 0 ? void 0 : _a.message) {
                    if (util_1.isArray((_b = exception === null || exception === void 0 ? void 0 : exception.response) === null || _b === void 0 ? void 0 : _b.message)) {
                        message = (_c = exception === null || exception === void 0 ? void 0 : exception.response) === null || _c === void 0 ? void 0 : _c.message[0];
                    }
                    else {
                        message = (_d = exception === null || exception === void 0 ? void 0 : exception.response) === null || _d === void 0 ? void 0 : _d.message;
                    }
                }
                else {
                    message =
                        this.extractMessageFromException(exception, contextType) || '';
                }
                return {
                    code: this.defaultValidationErrorPrefix,
                    message: message,
                    status: this.extractStatusCodeFromException(exception, common_1.HttpStatus.BAD_REQUEST),
                };
            }
            if (errorLayer === ErrorLayerEnum.EXCEPTION) {
                const { code, message, statusCode } = exception.exception;
                return {
                    code: code.toString(),
                    message: this.extractMessageFromException(exception.exception, contextType),
                    status: this.extractStatusCodeFromException(exception, statusCode),
                };
            }
            if (errorLayer === ErrorLayerEnum.APPLICATION) {
                if (exception instanceof common_1.HttpException) {
                    return {
                        code: this.defaultApplicationErrorPrefix,
                        message: this.extractMessageFromException(exception, contextType),
                        status: this.extractStatusCodeFromException(exception, exception.getStatus()),
                    };
                }
                return {
                    code: this.defaultApplicationErrorPrefix,
                    message: this.extractMessageFromException(exception, contextType),
                    status: this.extractStatusCodeFromException(exception, exception.status || common_1.HttpStatus.INTERNAL_SERVER_ERROR),
                };
            }
        }
        extractMessageFromException(exception, contextType) {
            if (contextType !== 'rpc') {
                if (this.isRpcException(exception)) {
                    return exception.details;
                }
                return exception.message;
            }
            return exception.message || 'There was an error';
        }
        extractStatusCodeFromException(exception, defaultStatusCode) {
            if (this.isRpcException(exception)) {
                return (RpcStatusToHttpStatusMap[exception.code] ||
                    common_1.HttpStatus.INTERNAL_SERVER_ERROR);
            }
            return defaultStatusCode;
        }
        isRpcException(exception) {
            if (exception.details && exception.code && exception.metadata) {
                return true;
            }
            return false;
        }
        handleRestContext(request, response, status, message, code) {
            return this.setResponseCode(response, status).send({
                message,
                code,
                statusCode: status,
                timestamp: new Date().toISOString(),
                path: request.url,
            });
        }
        handleRpcContext(code, message = '', status = common_1.HttpStatus.INTERNAL_SERVER_ERROR) {
            return rxjs_1.throwError({
                message: `${message} [${code}]`,
                code: this.getMappedRpcStatusFromHttpStatusCode(status),
            });
        }
        graphqlFormatError(status, message, code) {
            return new common_1.HttpException({
                message,
                code,
                statusCode: status,
            }, status);
        }
        identifyErrorLayer(exception) {
            if (exception instanceof typeorm_1.QueryFailedError ||
                exception.constructor.name === 'QueryFailedError') {
                return ErrorLayerEnum.DATABASE;
            }
            if (exception instanceof common_1.BadRequestException ||
                exception.constructor.name === 'BadRequestException') {
                return ErrorLayerEnum.VALIDATION;
            }
            if (exception instanceof mapped_exception_error_class_1.MappedExceptionError ||
                exception.constructor.name === 'MappedExceptionError') {
                return ErrorLayerEnum.EXCEPTION;
            }
            return ErrorLayerEnum.APPLICATION;
        }
        getMappedRpcStatusFromHttpStatusCode(httpStatus) {
            return HttpStatusToRpcStatusMap[httpStatus] || grpc_1.status.UNKNOWN;
        }
        setResponseCode(response, code) {
            if (response.code) {
                return response.code(code);
            }
            else if (response.status) {
                return response.status(code);
            }
            return response;
        }
    };
    MappedExceptionFilter = __decorate([
        common_1.Catch()
    ], MappedExceptionFilter);
    return MappedExceptionFilter;
})();
exports.MappedExceptionFilter = MappedExceptionFilter;
//# sourceMappingURL=mapped-exception.filter.js.map